#The D232/DUSB should never be exposed to the internet.
#For remote access, you must use a VPN.

#www.plasmatronics.com.au
#Plasmatronics 2020.10.20

import datetime
from pymodbus.client.sync import ModbusSerialClient

dSport = 'COM1' #'/dev/ttyS0'

#show ports:
#import serial.tools.list_ports as port_list
#ports = list(port_list.comports())
#for p in ports:
#    print (p)

#===================================================

def strDate( DayFraction ): #convert MicroSoft day fraction floats to str
    strDate = str(datetime.timedelta(days=DayFraction))

def scale_dingo_status(laFF00 , laFF01, laB2, errcode, sErr):
        
    #temp
    #batv, vmult,svdelta
    #voltIndex, statIndex
    #dim aData[31)

    aData = [None]*30   #starts at 3 to allign with dingo.xls columns
    volt = [12, 24, 32, 36, 48, 0]
    stat = ["boost", "equl", "absb", "float", "ERROR"]
    
    #shift lists (to maintain compatible indexes): 
    laFF00 =  [None]+ laFF00
    laFF01 =  [None]+ laFF01
    laB2   =  [None]+ laB2
    
    errCode = 0
    #aData[0) = Now()    

    statIndex = (laFF00[14] // 0x40) & 0x03


    if statIndex >= len(stat):
        statIndex = -1
        if (errcode == 0) :
            errcode = -1
            sErr = "Data Error"
    
    voltIndex = laFF00[1] & 0xF
    if voltIndex >= len(volt):
        voltIndex = 0
        if errcode == 0 :
            errcode = -2
            sErr = "Data Error"
    

    vmult = volt[voltIndex] // 12.0
    batv = ((laFF00[5]*256.0) + laFF00[6]) / 10.0
    

    temp = laFF00[4]                                    #battery temperature
    if temp == 128 : temp = ""

    aData[3] = stat[statIndex]    		#Regulator Mode state ((laFF00[14) // 0x40) & 0x3)
    aData[4] = batv     			#Battery voltage (laFF00[5), laFF00[6))
    aData[5] = vmult * laFF00[7] / 10        		#Regulation target voltage
    aData[6] = vmult * laFF00[12] / 10 			#Day max battery voltage
    aData[7] = vmult * laFF00[13] / 10 			#Day min battery voltage
    aData[8] = laFF00[11]        			#State of Charge
    aData[9]  =((laFF01[ 1] * 256) + laFF01[ 2] + (laFF01[ 9] * 256) + laFF01[10]) / 10 #charge current
    aData[10] = (laFF01[11] * 256) + laFF01[12]	#AH charge
    aData[11] =((laFF01[ 5] * 256) + laFF01[ 6] + (laFF01[13] * 256) + laFF01[14])/ 10 	#load current
    aData[12] = (laFF01[15] * 256) + laFF01[16]	#AH
    if laFF00[10] <= 241 :
     	aData[13] = strDate(laFF00[10] * 6 / 1440)  	#clock time we want
    
    aData[14] = laFF00[9]              		#Days since last boost
    aData[15] = laFF00[8]              		#Days since equalization
    aData[16] = temp				#temperature (laFF00[4))
    aData[17] = strDate(((laFF00[3] * 6) + laFF00[2]) / 1440) #clock
    aData[18] = volt[voltIndex]      		#VOLT setting value (laFF00[1] & 0xF)
    aData[19] = (laFF00[1] & 0xF0) // 0x10    #PROG setting value
        
    aData[21] = (laFF00[14]) & 1             #Load State
    aData[22] = (laFF00[14] // 0x2) & 1       #Generator State
    aData[23] = (laFF00[14] // 0x4) & 1       #Event controller state
    aData[24] = (laFF00[14] // 0x8) & 1       #Charge protection state
    aData[25] = (laFF00[14] // 0x10) & 1      #Alarm State
    aData[26] = (laFF00[14] // 0x20) & 1      #Load protection state
    aData[27] = (laFF00[14] // 0x40) & 0x3    #Regulator Mode state
    
    svdelta = (laB2[1] * 256) + laB2[2]        #solar voltage
    if svdelta < 28 :
        aData[29] = 0
    else :
        aData[29]= batv + (svdelta / 10)       #solar voltage
    
    return aData

#=== Start =========================================

client= ModbusSerialClient(method = 'rtu', port=dSport,stopbits = 1,timeout =1, bytesize = 8, parity = 'E', baudrate = '9600')
client.inter_char_timeout = 1
try:
    laFF00 = client.read_holding_registers(0XFF00,16,unit=8).registers
    laFF01 = client.read_holding_registers(0XFF01,16,unit=8).registers
    laB2   = client.read_holding_registers(0XB2,2,unit=8).registers

    errcode=0
    sErr=""
    aData = scale_dingo_status(laFF00 , laFF01, laB2, errcode, sErr)
    aData = aData[3:]   #discard the first 3 elements which are empty columns

    print(["Date","Vbat","Vtarget","Vmax","Vmin","State of Charge","Charge current",
       "Ah Daily Charge","Load Current","Ah Daily Load","Float time","Days since boost",
       "Days since eq","Battery temperature","clock","volt","prog","","load state",
       "generator state","event state","charge protection","alarm state",
       "load protection","charge status","","solar voltage"])

    print(aData)
finally:
    client.close()

